@@ -19,6 +19,29 @@ from plib.ui.textfile import PTextFile
 from plib.ui.widgets import *
 
 
+class PMemoFile(PTextFile):
+    
+    untitled = object()
+    
+    def __init__(self, app, control):
+        PTextFile.__init__(self, app, control, [
+            ("Text Files", ["txt"]),
+            ("All Files", ["*"]),
+        ])
+        self.main_window = app.main_window
+        self.base_title = app.main_title
+    
+    def set_filename(self, filename):
+        super(PMemoFile, self).set_filename(filename)
+        if filename:
+            self.main_window.set_caption("{} - {}".format(
+                self.base_title,
+                "<Untitled>" if filename is self.untitled else filename
+            ))
+        else:
+            self.main_window.set_caption(self.base_title)
+
+
 class PNotepad(PApplication):
     
     about_data = {
@@ -38,7 +61,7 @@ class PNotepad(PApplication):
     
     menu_actions = [
         (MENU_FILE, (
-            ACTION_FILE_NEW, ACTION_FILE_OPEN, ACTION_FILE_SAVE, ACTION_FILE_SAVEAS, ACTION_EXIT,
+            ACTION_FILE_NEW, ACTION_FILE_OPEN, ACTION_FILE_SAVE, ACTION_FILE_SAVEAS, ACTION_FILE_CLOSE, ACTION_EXIT,
         )),
         (MENU_EDIT, (
             ACTION_EDIT_UNDO, ACTION_EDIT_REDO,
@@ -106,12 +129,26 @@ class PNotepad(PApplication):
         for fenable in self.dirty_list:
             fenable(dirty)
     
+    filename_to_open = None
+    file_not_found = False
+    
+    def check_file_open(self):
+        if self.filename_to_open:
+            filename = self.filename_to_open
+            if os.path.isfile(filename):
+                print("Opening", filename)
+                self.memo_file.open_filename(filename)
+            else:
+                print(filename, "not found!")
+                self.file_not_found = True
+    
     def after_create(self):
-        self.memo_file = PTextFile(self, self.memo_notepad)
+        self.memo_file = PMemoFile(self, self.memo_notepad)
         self.enable_list, self.dirty_list = self.get_enable_list()
         self.enable_actions()
         self.update_filesize_status()
         self.update_overwrite_status()
+        self.check_file_open()
     
     def set_status_text(self, text):
         print(text)
@@ -127,6 +164,10 @@ class PNotepad(PApplication):
     def update_overwrite_status(self):
         self.label_overwrite.caption = self.overwrite_labels[self.memo_notepad.overwrite_mode]
     
+    def on_app_shown(self):
+        if self.file_not_found:
+            self.message_box.error("File Error", "File {} not found".format(self.filename_to_open))
+    
     def on_notepad_changed(self):
         self.update_filesize_status()
     
@@ -175,6 +216,10 @@ class PNotepad(PApplication):
             self.enable_actions()
             self.set_status_text("Saved file: {}".format(self.memo_file.filename))
     
+    def on_file_close(self):
+        self.memo_file.close_file()
+        self.update_filesize_status()
+    
     def on_edit_undo(self):
         self.memo_notepad.undo_last()
         self.enable_actions()
@@ -241,4 +286,6 @@ class PNotepad(PApplication):
 
 
 if __name__ == "__main__":
+    if len(sys.argv) > 1:
+        PNotepad.filename_to_open = sys.argv[1]
     PNotepad().run()
